# -*- coding: ascii -*-

###########################################################################
# clive, video extraction utility
# Copyright (C) 2007 Toni Gundogdu
#
# clive is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################

import sys
import os
import subprocess
import Tkinter
import tkSimpleDialog
import tkFileDialog
import tkMessageBox

from clive.opts import Options

__all__ = ['Configure']

class Configure:
	def __init__(self):
		self.widgets = {}
	
	def main(self):
		self.optparser = Options()
		(self.opts, self.args) = self.optparser.parse()
		self.root = Tkinter.Tk()
		self.root.title('clive - configure')
		self.root.geometry('+100+100')
		self._create_widgets()
		self._set_initial_values()
		self.root.mainloop()
	
	def _group(self, parent, group, rows):
		(g_text, g_row, g_col, g_sticky, g_colspan) = group

		g = Tkinter.LabelFrame(parent, text=g_text)
		g.grid(row=g_row, column=g_col, padx=10, pady=10,
			sticky=g_sticky, columnspan=g_colspan)

		g.grid_rowconfigure(0, weight=1)
		g.grid_columnconfigure(0, weight=1)

		for row in rows:
			(r_text, r_row, r_col, r_items, r_var, r_type) = row

			if r_text:
				l = Tkinter.Label(g, text=r_text)
				l.grid(row=r_row, column=r_col, sticky='w', padx=5, pady=5)

			for (index, item) in enumerate(r_items):
				(text, name, cmd) = item
				if r_type == 'radio':
					w = Tkinter.Radiobutton(g, text=text,
						variable=r_var, value=name, command=cmd)
				elif r_type == 'check':
					if not self.widgets.has_key(name):
						self.widgets[name] = Tkinter.IntVar()
					w = Tkinter.Checkbutton(g, text=text,
						variable=self.widgets[name], command=cmd)
				else:
					continue
				w.grid(row=r_row, column=r_col+index+1,
					padx=5, pady=5, sticky='w')

	def _create_widgets(self):
		self.root.rowconfigure(0, weight=1)
		self.root.columnconfigure(0, weight=1)

		items = [
			('Skip', '', None),
			('Overwrite', 'overwrite', None),
			('Rename', 'rename', None),
		]

		self.widgets['exists'] = Tkinter.StringVar()
		self._group(
			self.root,
			group = ('If output file exists:', 0, 0, 'nw', 1),
			rows = [
				(None, 0, 0, items, self.widgets['exists'], 'radio'),
			]
		)

		items = [
			('Verbose', 'verbose', None),
			('Log', 'log', None),
			('Read from clipboard', 'from_clipb', self._on_clipb_changed),
		]

		self._group(
			self.root,
			group = ('Options', 0, 1, 'ne', 1),
			rows = [
				(None, 0, 0, items, None, 'check'),
			]
		)

		items_play = [
			('Disabled', 'off', self._on_play_changed),
			('Source', 'src', self._on_play_changed),
			('MPEG', 'mpg', self._on_play_changed),
			('AVI', 'avi', self._on_play_changed),
		]

		items_encode = [
			('Disabled', 'off', self._on_encode_changed),
			('MPEG', 'mpg', self._on_encode_changed),
			('AVI', 'avi', self._on_encode_changed),
		]

		self.widgets['play'] = Tkinter.StringVar()
		self.widgets['re_encode'] = Tkinter.StringVar()

		self._group(
			self.root,
			group = ('Action after successful extraction:',
				1, 0, 'nwe', 2),
			rows = [
				('Play extracted video:', 0, 0, items_play,
					self.widgets['play'], 'radio'),
				('Re-encode extracted video:', 1, 0, items_encode,
					self.widgets['re_encode'], 'radio'),
			]
		)

		btns = Tkinter.Frame(self.root)
		btns.grid(row=2, column=1, padx=5, pady=5, sticky='e')

		btn_items = [
			('More...', self._on_more),
			('Cancel', self.root.destroy),
			('OK', self._on_exit),
		]

		for (index, btn) in enumerate(btn_items):
			(text, cmd) = btn
			b = Tkinter.Button(btns, text=text, command=cmd)
			b.grid(row=0, column=index, padx=5, pady=5, sticky='w')

	def _set_initial_values(self):
		action = '' # 'skip'
		if self.opts.overwrite: action = 'overwrite'
		elif self.opts.rename: action = 'rename'
		self.widgets['exists'].set(action)

		self.widgets['verbose'].set(self.opts.verbose)
		self.widgets['log'].set(self.opts.log)
		self.widgets['from_clipb'].set(self.opts.from_clipb)

		self.widgets['play'].set(self.opts.play)
		self.widgets['re_encode'].set(self.opts.re_encode)

	def _on_clipb_changed(self):
		if sys.platform != 'win32' and self.widgets['from_clipb'].get():
			if not self.opts.xclip or len(self.opts.xclip) == 0:
				tkMessageBox.showwarning('Warning',
					'Path to `xclip` needs to be specified. ' +
					'You can do so by clicking the "More..." button below.')

	def _on_play_changed(self):
		v = self.widgets['play'].get()
		if v != 'off':
			if not self.opts.player or len(self.opts.player) == 0:
				tkMessageBox.showwarning('Warning',
					'Path to a player command needs to be specified. ' +
					'You can do so by clicking the "More..." button below.')
			if v != 'src':
				self.widgets['re_encode'].set(v)

	def _on_encode_changed(self):
		p = self.widgets['play'].get()
		r = self.widgets['re_encode'].get()
		if r != 'off':
			if not self.opts.ffmpeg or len(self.opts.ffmpeg) == 0:
				tkMessageBox.showwarning('Warning',
					'Path to `ffmpeg` needs to be specified. ' +
					'You can do so by clicking the "More..." button below.')
			if p != 'off' and p != 'src':
				self.widgets['play'].set(r)

	def _on_more(self):
		MoreDialog(self.root, 'More', self.opts)

	def _on_exit(self):
		v = self.widgets['exists'].get()
		if v == 'overwrite': self.opts.overwrite = 1
		elif v == 'rename': self.opts.rename = 1
		else: self.opts.overwrite = 0; self.opts.rename = 0;

		self.opts.verbose = self.widgets['verbose'].get()
		self.opts.log = self.widgets['log'].get()
		self.opts.from_clipb = self.widgets['from_clipb'].get()

		self.opts.play = self.widgets['play'].get()
		self.opts.re_encode = self.widgets['re_encode'].get()

		self.optparser.write_config(self.opts)
		self.root.destroy()


class MoreDialog(tkSimpleDialog.Dialog):
	def __init__(self, parent, title, opts):
		self.opts = opts
		tkSimpleDialog.Dialog.__init__(self, parent, title)

	def _entry(self, parent, label, row, col=0,
			btn_cmd=None, btn_text='...'):

		l = Tkinter.Label(parent, text=label)
		l.grid(row=row, column=col, padx=5, pady=5, sticky='w')

		v = Tkinter.StringVar()
		e = Tkinter.Entry(parent, textvariable=v)
		e.grid(row=row, column=col+1, padx=5, pady=5, sticky='we')

		parent.rowconfigure(col+1, weight=1)
		parent.columnconfigure(col+1, weight=1)

		if btn_cmd:
			b = Tkinter.Button(parent, text=btn_text, command=btn_cmd)
			b.grid(row=row, column=col+2, padx=5, pady=5, sticky='e')

		return v

	def _group(self, parent, group, items):
		(g_text, g_row, g_col, g_sticky, g_colspan) = group

		g = Tkinter.LabelFrame(parent, text=g_text)
		g.grid(row=g_row, column=g_col, padx=10, pady=10,
			sticky=g_sticky, columnspan=g_colspan)

		for (index, item) in enumerate(items):
			(i_name, i_text, i_type, i_sticky, i_colspan, i_cmd) = item
			if i_type == 'entry':
				self.widgets[i_name] = self._entry(g,
					i_text, row=index, btn_cmd=i_cmd)
			elif i_type == 'check':
				v = Tkinter.IntVar()
				w = Tkinter.Checkbutton(g, text=i_text, variable=v)
				w.grid(row=index, column=0, padx=5, pady=5,
					columnspan=i_colspan, sticky=i_sticky)
				self.widgets[i_name] = v
			elif i_type == 'button':
				w = Tkinter.Button(g, text=i_text, command=i_cmd)
				w.grid(row=index, column=0, padx=5, pady=5,
					columnspan=i_colspan)

	def body(self, master):
		self.root = master
		self.widgets = {}
		self.result = None

		items = [
			('proxy', 'Proxy:', 'entry', None, 1, None),
			('user_agent', 'User-agent:', 'entry', None, 1, None),
			('throttle', 'Throttle limit (KB/s):', 'entry', None, 1, None),
			('gzip', 'Use HTTP stream compression', 'check', 'w', 2, None),
		]

		self._group(master,
			group = ('Connection', 0, 0, 'nw', 1),
			items = items
		)

		items = [
			('output_mask', 'Mask:', 'entry', None, 1, None),
			('output_fmt', 'Format:', 'entry', None, 1, None),
		]

		self._group(master,
			group = ('Output', 0, 1, 'ne', 1),
			items = items
		)

		items = [
			('prefix', 'Prefix:', 'entry', None, 1, self._on_browse_prefix),
			('player', 'Player:', 'entry', None, 1, self._on_browse_player),
			('ffmpeg', 'ffmpeg:', 'entry', None, 1, self._on_browse_ffmpeg),
		]

		if sys.platform != 'win32':
			item = ('xclip', 'xclip:', 'entry', None, 1,
				self._on_browse_xclip)
			items.append(item)
			item = (None,'Locate', 'button', None, 1,
				self._on_locate)
			items.append(item)

		self._group(master,
			group = ('Paths', 1, 0, 'we', 2),
			items = items
		)

		self._set_initial_values()

	def _set_initial_values(self):
		if self.opts.proxy:
			self.widgets['proxy'].set(self.opts.proxy)
		if not self.opts._random_user_agent:
			self.widgets['user_agent'].set(self.opts.user_agent)
		if self.opts.throttle > 0:
			self.widgets['throttle'].set(str(self.opts.throttle/1024))
		self.widgets['gzip'].set(self.opts.gzip)

		if self.opts.output_mask != 'A-Za-z0-9':
			self.widgets['output_mask'].set(self.opts.output_mask)
		if self.opts.output_fmt != '%t.%e':
			self.widgets['output_fmt'].set(self.opts.output_fmt)

		if self.opts.prefix != os.getcwd():
			self.widgets['prefix'].set(self.opts.prefix)
		if self.opts.player:
			self.widgets['player'].set(self.opts.player)
		if self.opts.ffmpeg:
			self.widgets['ffmpeg'].set(self.opts.ffmpeg)
		if sys.platform != 'win32':
			if self.opts.xclip:
				self.widgets['xclip'].set(self.opts.xclip)
	"""
	def validate(self):
		return 1
		#if len(self.e.get()) == 0:
		#	return 0
		#return 1
	"""
	
	def apply(self):
		self.opts.proxy = self.widgets['proxy'].get()

		v = self.widgets['user_agent'].get()
		if self.opts._random_user_agent and len(v) > 0:
			self.opts._random_user_agent = 0 # Reset flag
		self.opts.user_agent = v

		v = self.widgets['throttle'].get()

		if len(v) == 0: v = 0
		else: v = int(v)
		self.opts.throttle = v*1024
		self.opts.gzip = self.widgets['gzip'].get()

		self.opts.output_mask = self.widgets['output_mask'].get()
		self.opts.output_fmt = self.widgets['output_fmt'].get()

		self.opts.prefix = self.widgets['prefix'].get()
		self.opts.player = self.widgets['player'].get()
		self.opts.ffmpeg = self.widgets['ffmpeg'].get()
		if sys.platform != 'win32':
			self.opts.xclip = self.widgets['xclip'].get()

	def _on_browse_prefix(self):
		path = tkFileDialog.askdirectory(parent=self.root)
		if path:
			self.widgets['prefix'].set(path)

	def _on_browse_player(self):
		path = tkFileDialog.askopenfilename(parent=self.root)
		if path:
			self.widgets['player'].set(path + ' %i')

	def _on_browse_ffmpeg(self):
		path = tkFileDialog.askopenfilename(parent=self.root)
		if path:
			self.widgets['ffmpeg'].set(path + ' -y -i %i %o')

	def _on_browse_xclip(self):
		path = tkFileDialog.askopenfilename(parent=self.root)
		if path:
			self.widgets['xclip'].set(path + ' -o')

	def _on_locate(self):
		cmds = [
			('vlc', '%i', 'player'),
			('ffmpeg', '-y -i %i %o', 'ffmpeg'),
			('xclip', '-o', 'xclip'),
		]

		for cmd in cmds:
			pipe = subprocess.Popen('which %s 2>/dev/null' % cmd[0],
				shell=1, stdout=subprocess.PIPE).stdout
			data = pipe.readlines()
			if len(data) > 0:
				ln = data[0].rstrip('\n')
				if len(ln) > 0:
					self.widgets[cmd[2]].set(ln + ' %s' % cmd[1])


if __name__ == '__main__':
	Configure().main()


